@@ -4,14 +4,17 @@ Created on 14.04.2011
 
 @author: kirov
 '''
+from __future__ import absolute_import
 import os
 import uuid
 import csv
-import cStringIO
 import codecs
+from functools import cmp_to_key
 
 from django.conf import settings
 from django.db.models import Q, Count, Avg, Max, Min, Sum
+import six
+from six.moves import range
 
 try:
     import xlwt
@@ -31,13 +34,13 @@ class UnicodeWriter:
 
     def __init__(self, f, dialect=csv.excel, encoding="utf-8", **kwds):
         # Redirect output to a queue
-        self.queue = cStringIO.StringIO()
+        self.queue = six.StringIO()
         self.writer = csv.writer(self.queue, dialect=dialect, **kwds)
         self.stream = f
         self.encoder = codecs.getincrementalencoder(encoding)()
 
     def writerow(self, row):
-        self.writer.writerow([unicode(s).encode("utf-8") for s in row])
+        self.writer.writerow([six.text_type(s).encode("utf-8") for s in row])
         # Fetch UTF-8 output from the queue ...
         data = self.queue.getvalue()
         data = data.decode("utf-8")
@@ -444,7 +447,7 @@ class GroupingRecordProvider(object):
                     ws.write(self.getattr(
                         item, 'index') + 2, idx, v, data_style)
         # вывод итогов
-        if not isinstance(total, (int, long)):
+        if not isinstance(total, six.integer_types):
             total_row = total[1]
             for idx, k in enumerate(columns_cash):
                 if k == "grouping":
@@ -508,7 +511,7 @@ class GroupingRecordProvider(object):
                     row.append(v if v else '')
             ws.writerow(row)
         # вывод итогов
-        if not isinstance(total, (int, long)):
+        if not isinstance(total, six.integer_types):
             row = []
             total_row = total[1]
             for idx, k in enumerate(columns_cash):
@@ -639,11 +642,11 @@ class GroupingRecordModelProvider(GroupingRecordProvider):
             # сортировка
             sort_fields = []
             # оставлено для совместимости
-            if isinstance(sorting, dict) and sorting.keys():
-                for sort_field, sort_dir in sorting.iteritems():
+            if isinstance(sorting, dict) and list(sorting.keys()):
+                for sort_field, sort_dir in six.iteritems(sorting):
                     #необходимо исключить из сортировки поля, которые не входят в aggr, иначе по ним будет сделана ненужная группировка
-                    if sort_field in aggregates.keys() or sort_field == field or not field:
-                        if sort_field in aggregates.keys() and field:
+                    if sort_field in list(aggregates.keys()) or sort_field == field or not field:
+                        if sort_field in list(aggregates.keys()) and field:
                             # необходимо добавить суффикс к полю, которое в аггрегатах, иначе сортировать будет не по тому что надо :)
                             sort_field = sort_field + '__' + aggregates[sort_field]
                         if sort_dir == 'DESC':
@@ -654,8 +657,8 @@ class GroupingRecordModelProvider(GroupingRecordProvider):
             elif isinstance(sorting, list) and sorting:
                 for sort_field, sort_dir in sorting:
                     #необходимо исключить из сортировки поля, которые не входят в aggr, иначе по ним будет сделана ненужная группировка
-                    if sort_field in aggregates.keys() or sort_field == field or not field:
-                        if sort_field in aggregates.keys() and field:
+                    if sort_field in list(aggregates.keys()) or sort_field == field or not field:
+                        if sort_field in list(aggregates.keys()) and field:
                             # необходимо добавить суффикс к полю, которое в аггрегатах, иначе сортировать будет не по тому что надо :)
                             sort_field = sort_field + '__' + aggregates[sort_field]
                         if sort_dir == 'DESC':
@@ -715,9 +718,9 @@ class GroupingRecordModelProvider(GroupingRecordProvider):
             index = 0
             query = self.get_data()
             # оставлено для совместимости
-            if isinstance(sorting, dict) and sorting.keys():
+            if isinstance(sorting, dict) and list(sorting.keys()):
                 order = []
-                for sort_field, sort_dir in sorting.iteritems():
+                for sort_field, sort_dir in six.iteritems(sorting):
                     if sort_dir == 'DESC':
                         order.append('-' + sort_field)
                     else:
@@ -853,7 +856,7 @@ class GroupingRecordModelProvider(GroupingRecordProvider):
             # оставлено для совместимости
             if sorting:
                 if isinstance(sorting, dict):
-                    pairs = sorting.iteritems()
+                    pairs = six.iteritems(sorting)
                 elif isinstance(sorting, list):
                     pairs = sorting
                 else:
@@ -862,8 +865,8 @@ class GroupingRecordModelProvider(GroupingRecordProvider):
                     # необходимо исключить из сортировки поля,
                     # которые не входят в aggr,
                     # иначе по ним будет сделана ненужная группировка
-                    if sort_field in aggregates.keys() or sort_field == field:
-                        if sort_field in aggregates.keys():
+                    if sort_field in list(aggregates.keys()) or sort_field == field:
+                        if sort_field in list(aggregates.keys()):
                             # необходимо добавить суффикс к полю,
                             # которое в аггрегатах,
                             # иначе сортировать будет не по тому что надо :)
@@ -890,35 +893,28 @@ class GroupingRecordDataProvider(GroupingRecordProvider):
     Провайдер для массива
     '''
     def update_aggregates(self, aggregates, aggr_rec, rec):
+
+        sum_ = lambda val_aggr, val_curr: val_aggr + val_curr
+        type_map = {
+            'sum': sum_,
+            'count': lambda val_aggr, val_curr: val_aggr + 1,
+            'min': lambda val_aggr, val_curr: min(val_aggr, val_curr),
+            'max': lambda val_aggr, val_curr: max(val_aggr, val_curr),
+            'avg': sum_,  # Деление на количество реализуется в reader().
+        }
+
         for agg in aggregates.keys():
             agg_type = aggregates[agg]
-            agg_value = self.getattr(rec, agg)
-            if agg_type == 'sum':
-                aggr_rec[agg] = (
-                    (agg_value + aggr_rec[agg])
-                    if agg in aggr_rec else
-                    agg_value
-                )
-            elif agg_type == 'count':
-                aggr_rec[agg] = 1 + aggr_rec.get(agg, 0)
-            elif agg_type == 'min':
-                aggr_rec[agg] = min(
-                    agg_value,
-                    aggr_rec.get(agg, agg_value)
-                )
-            elif agg_type == 'max':
-                aggr_rec[agg] = max(
-                    agg_value,
-                    aggr_rec.get(agg, agg_value)
-                )
-            elif agg_type == 'avg':
-                aggr_rec[agg] = (
-                    (agg_value + aggr_rec[agg])
-                    if agg in aggr_rec else
-                    agg_value
-                )
-            elif callable(agg_type):
-                aggr_rec[agg] = agg_type(aggr_rec.get(agg), agg_value)
+
+            val_aggregated = aggr_rec.get(agg, 0)
+            val_current = self.getattr(rec, agg) or 0
+
+            if callable(agg_type):
+                func = agg_type
+            else:
+                func = type_map.get(agg_type)
+
+            aggr_rec[agg] = func(val_aggregated, val_current)
 
     def reader(
         self, grouped, offset, level_index, level_keys,
@@ -943,9 +939,9 @@ class GroupingRecordDataProvider(GroupingRecordProvider):
             self.setattr(item, 'lindex', None)
             self.setattr(item, 'count', count)
             if aggr_rec:
-                for agg, aggv in aggregates.iteritems():
+                for agg, aggv in six.iteritems(aggregates):
                     # для средних - посчитаем среднее
-                    if agg == 'avg':
+                    if aggv == 'avg':
                         self.setattr(
                             item, agg,
                             aggr_rec[agg] / self.getattr(item, 'count')
@@ -992,7 +988,7 @@ class GroupingRecordDataProvider(GroupingRecordProvider):
                 if found:
                     if field:
                         group_value = self.getattr(rec, field)
-                        if not group_value in level.keys():
+                        if not group_value in list(level.keys()):
                             level[group_value] = 1
                             aggr_rec = {}
                             aggregate_values[group_value] = aggr_rec
@@ -1015,7 +1011,7 @@ class GroupingRecordDataProvider(GroupingRecordProvider):
                     self.setattr(item, 'id', i)
                     self.setattr(item, 'indent', level_index)
                     self.setattr(item, 'count', level[i])
-                    for agg, aggv in aggregates.iteritems():
+                    for agg, aggv in six.iteritems(aggregates):
                         # для средних - посчитаем среднее
                         if agg == 'avg':
                             self.setattr(
@@ -1071,9 +1067,9 @@ class GroupingRecordDataProvider(GroupingRecordProvider):
 
         # а вот теперь сортируем и граничиваем
         # оставлено для совместимости
-        if isinstance(sorting, dict) and sorting.keys():
+        if isinstance(sorting, dict) and list(sorting.keys()):
             order = []
-            for sort_field, sort_dir in sorting.iteritems():
+            for sort_field, sort_dir in six.iteritems(sorting):
                 if sort_dir == 'DESC':
                     order.append('-' + sort_field)
                 else:
@@ -1200,7 +1196,7 @@ class GroupingRecordDataProvider(GroupingRecordProvider):
                 # если успешно проверили все поля, то значит это наша запись
                 if finded:
                     group_value = self.getattr(rec, field)
-                    if not group_value in level.keys():
+                    if not group_value in list(level.keys()):
                         level[group_value] = 1
                         aggr_rec = {}
                         aggregate_values[group_value] = aggr_rec
@@ -1253,9 +1249,9 @@ class GroupingRecordDataProvider(GroupingRecordProvider):
 
             # а вот теперь сортируем и граничиваем
             # оставлено для совместимости
-            if isinstance(sorting, dict) and sorting.keys():
+            if isinstance(sorting, dict) and list(sorting.keys()):
                 order = []
-                for sort_field, sort_dir in sorting.iteritems():
+                for sort_field, sort_dir in six.iteritems(sorting):
                     if sort_dir == 'DESC':
                         order.append('-' + sort_field)
                     else:
@@ -1288,9 +1284,12 @@ class GroupingRecordDataProvider(GroupingRecordProvider):
                 (col.strip(), 1)
             ) for col in columns
         ]
+
+        compare = lambda a, b: (a > b) - (a < b)
+
         def comparer(left, right):
             for attr, mult in comparers:
-                result = cmp(
+                result = compare(
                     self.getattr(left, attr),
                     self.getattr(right, attr)
                 )
@@ -1298,4 +1297,4 @@ class GroupingRecordDataProvider(GroupingRecordProvider):
                     return mult * result
             else:
                 return 0
-        return sorted(items, cmp=comparer)
+        return sorted(items, key=cmp_to_key(comparer))
