@@ -1,24 +1,30 @@
-#coding: utf-8
+# coding: utf-8
 '''
 Created on 14.04.2010
 
 @author: airat
 '''
 
-import decimal
-import time
-import datetime
-import re
+from __future__ import absolute_import
 import codecs
-import uuid
+import datetime
+import decimal
+import re
 import string
+import time
+import uuid
 import xml.dom.minidom
 
-from django.utils.encoding import smart_unicode
-from django.db import models, transaction
+from django.db import models
+from django.utils.encoding import smart_str
+from m3_django_compat import atomic
+import six
+from six.moves import range
+from six.moves import zip
+
 
 try:
-    from dbfpy.dbf import Dbf
+    from dbfpy3.dbf import Dbf
 except ImportError:
     # Заглушка на случай неустановленного dbfpy
     class Dbf(object):
@@ -56,7 +62,7 @@ def read_simple_CSV_dict_file(filename, comma=','):
         каждой записи в порядке, указанном в первой строке.
         Струткура возвращаемого значения аналогична той, что с методом @see read_simple_dict_file(filename)
     '''
-    assert isinstance(filename, basestring), u"filename must be 'str'"
+    assert isinstance(filename, six.string_types), u"filename must be 'str'"
 
     attrs = []
     values = []
@@ -67,9 +73,9 @@ def read_simple_CSV_dict_file(filename, comma=','):
             first_string = strip_bom(first_string)
 
 
-            attrs = [item.strip().lower() for item in smart_unicode(first_string).split(comma)]
+            attrs = [item.strip().lower() for item in smart_str(first_string).split(comma)]
             for num, line in enumerate(f, 1):
-                row_values = [item.strip() for item in smart_unicode(line).split(comma)]
+                row_values = [item.strip() for item in smart_str(line).split(comma)]
                 if len(row_values) == len(attrs):
                     values.append(row_values)
                 else:
@@ -83,7 +89,7 @@ def read_simple_DBF_dict_file(filename, encoding='utf-8',
                               code_field="CODE", name_field="NAME"):
     ''' Чтение данных из файла формата "DBase" (.dbf) '''
 
-    assert isinstance(filename, basestring), u"filename must be 'str'"
+    assert isinstance(filename, six.string_types), u"filename must be 'str'"
 
     attrs = []
     values = []
@@ -97,7 +103,7 @@ def read_simple_DBF_dict_file(filename, encoding='utf-8',
             for record in records:
                 vals_here = []
                 for name in attrs:
-                    vals_here.append(smart_unicode(record[name], encoding=encoding))
+                    vals_here.append(smart_str(record[name], encoding=encoding))
 
                 values.append(vals_here)
 
@@ -128,7 +134,7 @@ def read_simple_xml_file(filename, fields={}):
     fields: ключ - атрибут в узле, соответствуюший полю в таблице
     значение - поле в таблице
     '''
-    assert isinstance(filename, basestring), u"filename must be basestring type"
+    assert isinstance(filename, six.string_types), u"filename must be basestring type"
 
     values = []
     attrs = []
@@ -141,7 +147,7 @@ def read_simple_xml_file(filename, fields={}):
                 if attr in fields:
                     if not fields[attr] in attrs:
                         attrs.append(fields[attr])
-                    values_list.append(smart_unicode(node.attributes._attrs[attr].nodeValue, encoding=doc.encoding))
+                    values_list.append(smart_str(node.attributes._attrs[attr].nodeValue, encoding=doc.encoding))
             values.append(values_list)
     else:
         raise DictLoadException(None, u'Исходные данные отсутствуют. Пустой XML-файл')
@@ -160,7 +166,7 @@ def read_simple_dict_file(filename):
     )
     '''
 
-    assert isinstance(filename, basestring), u"filename must be 'str'"
+    assert isinstance(filename, six.string_types), u"filename must be 'str'"
 
     attrs = []
     values = []
@@ -173,7 +179,7 @@ def read_simple_dict_file(filename):
             else:
                 raise DictLoadException(None, u'Первая строка скрипта загрузки линейного справочника должна содержать название аттрибутов и начинаться с символа #')
 
-            attrs = [item.strip() for item in smart_unicode(first_string).split('\t')]
+            attrs = [item.strip() for item in smart_str(first_string).split('\t')]
             for num, line in enumerate(f, 1):
                 row_values = [item.strip() for item in line.split('\t')]
                 if len(row_values) == len(attrs):
@@ -195,7 +201,7 @@ def read_tree_dict_file(filename):
     )
     '''
 
-    assert isinstance(filename, basestring), u"filename must be 'str'"
+    assert isinstance(filename, six.string_types), u"filename must be 'str'"
 
     # паттерн для строки модели групп
     VALID_BRANCH = '\[\+\](\w| )+'
@@ -260,8 +266,8 @@ def read_tree_dict_file(filename):
             else:
                 raise DictLoadException(None, u'Вторая строка скрипта загрузки иерархического справочника должна содержать название аттрибутов модели линейного справочника и начинаться с символа #')
 
-            tree_attrs = [item.strip() for item in smart_unicode(first_string).split('\t')]
-            dict_attrs = [item.strip() for item in smart_unicode(second_string).split('\t')]
+            tree_attrs = [item.strip() for item in smart_str(first_string).split('\t')]
+            dict_attrs = [item.strip() for item in smart_str(second_string).split('\t')]
             dict_rows = {}
             tree_rows = {}
             parents = []
@@ -297,7 +303,8 @@ def read_tree_dict_file(filename):
 
     return (tree_attrs, dict_attrs, tree_rows, dict_rows)
 
-@transaction.commit_on_success
+
+@atomic
 def fill_simple_dict(model, data):
     '''
     Выполняет заполнение модели model (тип параметра - classobj) данными data,
@@ -317,7 +324,7 @@ def fill_simple_dict(model, data):
         for attr, val in zip(attrs, value):
             if val != u'None':
                 try:
-                    if fields.has_key(attr):
+                    if attr in fields:
                         converted = _convert_value(fields[attr], val)
                         setattr(obj, attr, converted)
                 except Exception as e:
@@ -327,7 +334,8 @@ def fill_simple_dict(model, data):
         except Exception as exc:
             raise DictLoadException(model.__name__, u'Не удалось сохранить запись справочника: %s' % string.join(value))
 
-@transaction.commit_on_success
+
+@atomic
 def fill_tree_dict(group_model, list_model, group_link, list_link, data):
 
     assert issubclass(list_model, models.Model), 'model must be subclass of django.db.models.Model'
