@@ -4,6 +4,7 @@ Created on 16.01.2011
 
 @author: akvarats
 """
+from __future__ import absolute_import
 import datetime
 
 try:
@@ -19,7 +20,7 @@ from django.contrib import auth
 try:
     from django.utils.log import logger
 except ImportError:
-    from django.utils.log import getLogger
+    from logging import getLogger
     logger = getLogger('django')
 
 
